
// ****************************** Tasten und Menues auswerten  *******************************************************************
// * Lesen der Drucktaster und zeige das richtige Menü an, wenn  auf den ensprechenden Taster geklickt wird.
// * Jeder Punkt in der Schalter-Anweisung ist ein Menü. Für jedes Menü werden die Schaltflächen Aktionen behandelt.
// *******************************************************************************************************************************
//*****************************************Funktion Tasten auslesen **************************************************************
void readButtons() 
{
  static unsigned long press_time;
  static bool cancelNextEnter;
  static bool detectingLongPress;

  //********************************************************* Hole Tasten status ************************************************  
  bool up_pressed = (button_UP.update() && button_UP.fell());             //von rose in fell geändert  ##########################
  bool down_pressed = (button_DOWN.update() && button_DOWN.fell());       //von rose in fell geändert  ##########################
  bool enter_pressed = false;
  bool enter_long_pressed = false;

//*************************** Dann kann die Enter-Taste auch länger gedrückt werden, um in das Kalibriermenü zu gelangen.*******
  if (button_ENTER.update()) 
  {
    if (button_ENTER.fell())          //von rose in fell geändert ################################################################
    {
      press_time = millis();         // ************  Erinnern Sie sich, wann die Taste gedrückt wurde. **************************
      detectingLongPress = true;     // ************ Start dedektierung für langen Tastendruck *********************************** 
    } 
    else if (button_ENTER.rose())     //von fell in rose geändert ################################################################
    { 
      if (!cancelNextEnter) 
        enter_pressed = true;                 
      detectingLongPress = false;
      cancelNextEnter = false;
    }      
  } 
  else if (detectingLongPress && (millis() - press_time) > 2000) 
  {
    enter_long_pressed = true;  // ************* Langer Tastendruck ***********
    cancelNextEnter = true;     // Abbruch der Eingabe bei Loslassen der Taste.
    detectingLongPress = false; // Ende der Detektierung des langen Tastendruckes   
  }

  
//********************************************************************************************************************************         
  if (up_pressed || enter_pressed || down_pressed)
  {
    switch(current_menu)
    {
      case MENU_MESSUNGEN:
        if (up_pressed) 
          current_menu = MENU_MIN_LEISTUNG;
        else if (down_pressed) 
          current_menu = MENU_MODULATIONINDEX;
        else if (enter_pressed) 
        {
          current_menu = MENU_FREQUENZ_SETZEN;
          display_line = 1; //****************** Setze Cursor auf Zeile 1
          display_col = 0; //******************* Setze Cursor auf Spalte 0
          menu_freq = selected_freq;          
          displayFrequencySettingsMenu();
        }
        break;
        //*****************************************************
      case MENU_MIN_LEISTUNG:
        if (up_pressed) 
          current_menu = MENU_MAX_LEISTUNG;
        else if (down_pressed) 
          current_menu = MENU_MESSUNGEN;
        else if (enter_pressed) 
        {       
          resetHistoryRingBuffers();                         
          displayMinMaxReset();
        }
        break;
        //*****************************************************
      case MENU_MAX_LEISTUNG:
        if (up_pressed) 
          current_menu = MENU_MODULATIONINDEX;
        else if (down_pressed) 
          current_menu = MENU_MIN_LEISTUNG;
        else if (enter_pressed) 
        {        
          resetHistoryRingBuffers();        
          displayMinMaxReset();
        }
        break;
        //*****************************************************
     case MENU_MODULATIONINDEX:
        if (up_pressed) 
          current_menu = MENU_MESSUNGEN;
        else if (down_pressed) 
          current_menu = MENU_MAX_LEISTUNG;        
        break;
        //*****************************************************
      case MENU_FREQUENZ_SETZEN:
        if (up_pressed) 
        {          
          // If on frequenz line            
          if (display_col == 0)          
            menu_freq+=1000;    // Add 1000MHz
          else if (display_col == 1)          
            menu_freq+=100;     // Add 100MHz       
          else if (display_col == 2)          
            menu_freq+=10;      // Add 10MHz       

          if (menu_freq > MAX_FREQUENZ)
            menu_freq = MAX_FREQUENZ;                 //************** Kann nicht höher als MAX_FREQUENZ sein. *******************
                    
          displayFrequencySettingsMenu();        
        }
        else if (down_pressed) 
        {          
          // If on frequenz line            
          if (display_col == 0)          
            menu_freq-=1000;  // Subtract 1000MHz
          else if (display_col == 1)          
            menu_freq-=100;   // Subtract 100MHz          
          else if (display_col == 2)          
            menu_freq-=10;    // Subtract 10MHz          

          if (menu_freq < MIN_FREQUENZ) 
            menu_freq = MIN_FREQUENZ;                 //*************** Kann nicht kleiner als MIN_FREQUENZ sein.*****************
            
          displayFrequencySettingsMenu();     
        }
        else if (enter_pressed) 
        {          
          if (display_col == 0) {
            display_col = 1;
            displayFrequencySettingsMenu();
          }
          else if (display_col == 1) {
            display_col = 2;
            displayFrequencySettingsMenu();
          }
          else if (display_col == 2) 
          {
            current_menu = MENU_ABSCHWAECHER_SETZEN;    
            display_line = 1;                           //******** Setze Cursor auf Zeile 1
            display_col = 0;                            //******** Setze Cursor auf Spalte 0      
            selected_freq = menu_freq;                  //******** Speichere als Standartfrequenz    
            attenuation = calculateAttenuation();       //******** Berechnung der vordämpfung
            dbm_at_0V = interpolatedDbmValue();         //******** Berechne neu dbm_at_0V
            displayAttenuationSettingsMenu();
          }
        }
        break;
        //*****************************************************
      case MENU_ABSCHWAECHER_SETZEN:
        if (up_pressed) 
        {
          if (display_col == 0) 
          {
            // If on profile row          
            setAttenuationProfile(true);                    //**************** Setze auf die nächste Frequenz in der Liste
          }
          else 
          { 
            // On attenuation line                        
            if (display_col == 7)
              custom_attenuation+=10;                       //**************** Schrittweite erhöhung derDämpfung mit 10
            else if (display_col == 8)
              custom_attenuation++;                         //**************** Schrittweite erhöhung derDämpfung mit 1
            else if (display_col == 10)
              custom_attenuation+=0.1;                      //**************** Schrittweite erhöhung derDämpfung mit 0.1
              
            if (custom_attenuation > MAX_ATTENUATION_DBM) 
              custom_attenuation = MAX_ATTENUATION_DBM;     //**************** Kann nicht höher seine als MAX_ATTENUATION_DBM;
          }            
          displayAttenuationSettingsMenu();        
        }
        else if (down_pressed) 
        {
          if (display_col == 0) 
          {
            // If on profile row          
            setAttenuationProfile(false);                   //****************  Setze auf die nächste Frequenz in der Liste
          }
          else 
          { 
            //****************************************************************** Auf der Abschächerzeile                        
            if (display_col == 7)
              custom_attenuation-=10;                       //***************** Erhöhe um den Faktor 10
            else if (display_col == 8)
              custom_attenuation--;                         //***************** Erhöhe um den Faktor 1
            else if (display_col == 10)
              custom_attenuation-=0.1;                      //***************** Erhöhe um den Faktor 0.1
              
            if (custom_attenuation < 0) 
              custom_attenuation = 0;                       //***************** Kann nicht kleiner als null sein
          }  
          displayAttenuationSettingsMenu();      
        }
        else if (enter_pressed) 
        {
          if (att_profile == 0)                             //*************** Anwender Profil
          {
            if (display_col == 0)                           //************* Wenn auf Profilauswahl-Spalte
            {
              display_col = 7;                              //************* Gehe zu  x10 Reihe         
            }
            else if (display_col == 7)                       //******** Wenn auf Dämpfung x10 Spalte
            {            
              display_col = 8;                              //************* Gehe zu x1 Reihe  
            }
            else if (display_col == 8)                      //******** Wenn auf Dämpfung x1 Spalte
            {            
              display_col = 10;                             //************* Gehe zu  x0.1 Reihe  
            }
            else if (display_col == 10)                     //******* Wenn auf Dämpfung x0.1 Spalte
            {
              writeSettingsToEEPROM();  //******* Schreiben Sie die Einstellungen in das EEPROM, damit wir beim nächsten Mal die gleichen Einstellungen verwenden können.
              //******************************** Gehe zurück ins  measure menu;
              current_menu = MENU_MESSUNGEN;              
            }
          }
          else //******************************* Wenn auf einem anderen Profil, dann custom'
          {
            writeSettingsToEEPROM();  //******** Schreiben der Einstellungen in das EEPROM, damit wir beim nächsten Mal die gleichen Einstellungen verwenden können.
            //********************************** Gehe zurück zum measure menu;
            current_menu = MENU_MESSUNGEN;
          }

          attenuation = calculateAttenuation();
          displayAttenuationSettingsMenu();
        }
        break;  
        //*****************************************************    
      case MENU_CALIBRIERUNG_AUSWAHL_FREQ:
        if (up_pressed) 
        {
          nextCalibrationPoint();                    
          displayCalibrateSelectedFreqMenu();           
        }
        else if (down_pressed) 
        {         
          previousCalibrationPoint();        
          displayCalibrateSelectedFreqMenu(); 
        }
        else if (enter_pressed)
        {
          if (selected_calibration_point == -2)
          {
            current_menu = MENU_MESSUNGEN;                                            //************* Zurück zu measurements            
          }
          else if (selected_calibration_point == -1)                                  //*****  Hinzufügen Kalibrierung
          {
            if (calibrationSlotLeft())
            {
              // Add new calibration point
              current_menu = MENU_CALIBRIERUNG_SETZE_FREQ;
              menu_freq = 100;                            // Setze 100MHz als beim Start mit..
              display_line = 1;                           // Setze Cursor auf Zeile 1
              display_col = 0;                            // Setze Cursor on Spalte 0                       
              displayFrequencySettingsMenu();
            }
            else
            {
              //******************************************** Kein Speicherplatz mehr vorhanden, kehren Sie zum Menü"Add new" zurück.
              displayNoCalibrationSlotLeft();
              selected_calibration_point = -1;
              current_menu = MENU_CALIBRIERUNG_AUSWAHL_FREQ;                    
              displayCalibrateSelectedFreqMenu();
            }
          }
          else
          {
            //******************************************** Lösche ausgewählte Kalibrierpunkte
            current_menu = MENU_CALIBRIERUNG_BEST_LOESCHEN;
            displayCalibrateConfirmDeleteMenu();
          }
        }        
        break;
        //*****************************************************
      case MENU_CALIBRIERUNG_BEST_LOESCHEN:
        if (up_pressed || down_pressed) 
        {
          current_menu = MENU_CALIBRIERUNG_AUSWAHL_FREQ;                      //******** Zurück zum vorherigen Menu;          
          displayCalibrateSelectedFreqMenu();
        }
        else
        {
          displayCalibrateDeletedMenuNotification();
          deleteCalibrationPoint(selected_calibration_point);          
          delay(1500);
          current_menu = MENU_CALIBRIERUNG_AUSWAHL_FREQ;     
          selected_calibration_point = -1;                                      //******************** Show add new menu (Neues Menü hinzufügen)               
          displayCalibrateSelectedFreqMenu();
        }        
        break;
        //*****************************************************
      case MENU_CALIBRIERUNG_SETZE_FREQ:
        if (up_pressed) 
        {          
          // If on frequenz line            
          if (display_col == 0)          
            menu_freq+=1000;       // Add 1000MHz
          else if (display_col == 1)          
            menu_freq+=100;  // Add 100MHz       
          else if (display_col == 2)          
            menu_freq+=10;         // Add 10MHz       

          if (menu_freq > MAX_FREQUENZ)
            menu_freq = MAX_FREQUENZ;                                         //******************** Kann nicht höher als die MAX_FREQUENZ
          
          displayFrequencySettingsMenu();        
        }
        else if (down_pressed) 
        {          
          // If on frequenz line            
          if (display_col == 0)          
            menu_freq-=1000;                                                    //**************** Abzählen  1000MHz
          else if (display_col == 1)          
            menu_freq-=100;                                                     //**************** Abzählen 100MHz          
          else if (display_col == 2)          
            menu_freq-=10;                                                          //**************** Abzählen 10MHz          

          if (menu_freq < MIN_FREQUENZ) 
            menu_freq = MIN_FREQUENZ;                                           //***** Kann nicht kleiner als MIN_FREQUENZ
            
          displayFrequencySettingsMenu();     
        }
        else if (enter_pressed) 
        {          
          if (display_col == 0) {
            display_col = 1;
            displayFrequencySettingsMenu();
          }
          else if (display_col == 1) {
            display_col = 2;
            displayFrequencySettingsMenu();
          }
          else if (display_col == 2) 
          {
            if (calibrationExists(menu_freq))
            {              
              //******************************** Kalibrierung bereits vorhanden.
              displayCalibrationExistsNotification(menu_freq);
              delay(1500);
              //******************************** Gehe zurück zu  'Add new'.
              selected_calibration_point = -1;
              current_menu = MENU_CALIBRIERUNG_AUSWAHL_FREQ;                    
              displayCalibrateSelectedFreqMenu();           
            }
            else
            {
              current_menu = MENU_CALIBRIERUNG_FRAGE_VERBUN;                
              displayCalibrateAskConnectMenu();
            }
          }
        }
        break;
        //*****************************************************
      case MENU_CALIBRIERUNG_FRAGE_VERBUN:
        if (up_pressed || down_pressed)
        {
          current_menu = MENU_CALIBRIERUNG_AUSWAHL_FREQ;                              //****** Zurück
          displayCalibrateSelectedFreqMenu();
        }
        else if (enter_pressed)
        {
          current_menu = MENU_CALIBRIERUNG_EINGA_LEISTUNG;  
          calibrated_power = -10;                                                     //**************************** Reset
          display_col = 8;
          display_line = 1;
          displayCalibrateEnterMeasuredPower();
        }
        break;
        //*****************************************************
      case MENU_CALIBRIERUNG_EINGA_LEISTUNG:
        if (up_pressed) 
        {          
          // On attenuation line                        
          if (display_col == 8)
            calibrated_power+=10;                                                     //********** Aufwärts zählen Abschwächer mit Fator 10
          else if (display_col == 9)
            calibrated_power++;                                                       //********** Aufwärts zählen Abschwächer mit Fator 1
          else if (display_col == 11)
            calibrated_power+=0.1;                                                    //********** Aufwärts zählen Abschwächer mit Fator 0.1
              
          if (calibrated_power > -10) 
            calibrated_power = -10; //********* Kann nicht höher als  -10 dBm;                     
          displayCalibrateEnterMeasuredPower();        
        }
        else if (down_pressed) 
        {          
          //*********************************** Auf der Dämpfungszeile                        
          if (display_col == 8)
            calibrated_power-=10;                                                     //********* Abwärts zählen Abschwächer mit Fator 10
          else if (display_col == 9)
            calibrated_power--;                                                       //********** Abwärts zählen Abschwächer mit Fator 1
          else if (display_col == 11)
            calibrated_power-=0.1;                                                    //******** Abwärts zählen Abschwächer mit Fator 0.1
              
          if (calibrated_power < -50) 
            calibrated_power = -50;                                                   //******** Kann nicht kleiner sein als -50 dBm;
           
          displayCalibrateEnterMeasuredPower();
        }
        else if (enter_pressed) 
        {          
          if (display_col == 8) //*********** Wenn auf Abschwächer x10 column
          {            
            display_col = 9;    // Gehe zu  x1 row  
            displayCalibrateEnterMeasuredPower();
          }
          else if (display_col == 9) //***** Wenn auf Abschwächer  x1 column
          {            
            display_col = 11;   //********** Gehe zu  x0.1 row  
            displayCalibrateEnterMeasuredPower();
          }
          else if (display_col == 11) //**** Wenn auf Abschwächer  x0.1 column
          {                       
            displayCalibrationSavedNotification();                      
            saveCalibrationData();
            mergeCalibrationData();           
            dbm_at_0V = interpolatedDbmValue();             
            current_menu = MENU_CALIBRIERUNG_AUSWAHL_FREQ;       
            selected_calibration_point = -1;                                          //********** Zeige hinzugefügtes neues Menu ************** 
            displayCalibrateSelectedFreqMenu();           
          }                   
        }
        break;    
        //*****************************************************  
    }                
  } 
  else if (enter_long_pressed) //*********************** Überprüfe ob die Taste lang gedrückt wurde 
  {
    //************************************************** Auswahl calibration menu
    current_menu =MENU_CALIBRIERUNG;    
    displayCalibrateFrequencyMenu();    
    //************************************************* Nächstes Menu nach 1500ms
    delay(1500);
    selected_calibration_point = -1;                                                  //**************** reset
    current_menu = MENU_CALIBRIERUNG_AUSWAHL_FREQ;
    displayCalibrateSelectedFreqMenu();
  }
}

// *********************************************** Funtion,  Warten Sie auf x ms, aber Abfragen ob Tastendruck in der Zwischenzeit.******************
void readButtonsAndWait(int waitMs)
{
  unsigned long t = millis();
  while (millis() - t < waitMs)
    readButtons();
}

int blinkCnt = 0;
// *********************************************** Funktion  Display Warnnachricht **********************************************************
void displayOverloadWarning()  //############################################################################################################
{
    displayloeschen();  
    display.setTextSize(2);  
    display.print(F(" OVERLOAD"));
    display.setCursor(0, 15);  
    display.print(F("  WARNING"));         
    display.setCursor(0, 30);       
    display.print(F("DISCONNECT"));
    display.setCursor(0, 45);
    display.print(F("   NOW! "));        
    display.display();
}

//* *********************Funktion, Anzeige der Einstellungen und Primärmessungen,  Frequenz, Abschwächer und Leistung in dBm und mW
void displayMeasurements() 
{
  static byte cnt = 0;
  byte chars = 0;
  displayloeschen();

  //*********************************************** Anzeigefrequenz und -dämpfung in der ersten Zeile alternat
  if (cnt <= 5)
  {    
    chars += display.print(F("Freq: "));    
    display.setTextSize(2);
    chars += display.print(selected_freq);  
    display.setTextSize(1);  
    chars += display.print(FS(str_MHz));   
    display.display(); 
  }
  else if (cnt <= 10)
  {
    chars += display.print(F("ATTN: "));   
    display.setTextSize(2);
    chars += printFormattedNumber(attenuation, 1, 1, false, false);    
    display.setTextSize(1);
    chars += display.print(FS(str_dB));  
    display.display();      
  }
  else if (cnt <= 15)
  {
    chars += display.print(F("Error: "));   
    display.setTextSize(2);
    chars += printFormattedNumber(currentError(), 1, 1, false, false);    
    display.setTextSize(1);
    chars += display.print(FS(str_dB));   
    display.display();     
  }
  cnt++;
  if (cnt > 15 || (cnt > 10 && currentError() == 0) )
    cnt = 0; // reset loop
  
  // Line 2
  display.setCursor(0, 25); // goto next line     
  if (power_notconnected)
    {     
    display.setTextSize(1);  
    display.print(F("INPUT TOO LOW/NC"));     // Show not connected
    display.display();
    }
  else
  {      
    // Show power measurement in dBm and watt.
    float actualPowerdBm = roundDecimals(avg_dbm + attenuation, 1);
    float actualPowerMw = convertDbmToMilliWatt(actualPowerdBm);    
    display.setTextSize(2);   
    display.setCursor(0, 25);
    chars = printFormattedNumber(actualPowerdBm, 2, 1, true, true);     //###############################################
    chars += display.print(FS(str_dBm));  
    dbm = 1;
    gross = 1;
    chars += printPowerWatts(actualPowerMw);
    dbm = 0;
    display.display();
  }
}

//****************************************************************Funktion  Anzeige der Maximalleistungsmessung der letzten 10s******************
void displayMinPower () 
{
    displayloeschen();
    
    if (power_notconnected) 
    {       
      display.print(F("Minumum power:  "));
      display.setCursor(0, 20);                                     //************************************ gehe zur nächsten Zeile    
      display.print(F("INPUT TOO LOW/NC"));
      display.display();
    }
    else 
    {      
      byte chars = display.print(F("Minimum power: "));    
      display.setCursor(0, 20);                                     //************************************ gehe zur nächsten Zeile
      float dBm = roundDecimals(min_dbm + attenuation, 1);   
      chars = printFormattedNumber(dBm, 1, 1, true, true);    
      chars += display.print(FS(str_dBm));  
      chars += display.print(" ");  
      chars += printPowerWatts(convertDbmToMilliWatt(dBm));
      display.display();    
    }
}

// ************************************************************Funktion,  Zeige die maximale Leistung *********************************************
void displayMaxPower () 
{
    displayloeschen();
    
    if (power_notconnected) 
    {       
      display.print(F("Maximum power:  "));
      display.setCursor(0, 20);                                   //************************************ gehe zur nächsten Zeile   
      display.print(F("INPUT TOO LOW/NC"));
      display.display();
    }
    else 
    {
      display.print(F("Maximum power:  "));           
      display.setCursor(0, 20);                                   //************************************ gehe zur nächsten Zeile    
      float dBm = roundDecimals(max_dbm + attenuation, 1);
      byte chars = printFormattedNumber(dBm, 1, 1, true, true);    
      chars += display.print(FS(str_dBm));  
      chars += display.print(" ");  
      chars += printPowerWatts(convertDbmToMilliWatt(dBm));
      display.display();    
    }
}

//**************************************************************Funktion,  Anzeige min. max. Reset-Benachrichtigung *********************************
void displayMinMaxReset() 
{
  displayloeschen();
  display.print(F("Min./Max."));
  display.setCursor(0, 20);                                       //************************************ gehe zur nächsten Zeile    
  display.print(F("ist gewesen Reset!"));
  display.display();
  delay(1000);
}

//**************************************************************Funktion,  Anzeige von PEP (Peak Envelope Power) und AM-Modulationsindex *************
void displayPEPModulationIndex () 
{
    displayloeschen();
            
    if (power_notconnected) 
    {       
      byte chars = display.print(F("PEP: "));
      display.setCursor(0, 20);                                   //************************************ gehe zur nächsten Zeile    
      chars = display.print(F("INPUT TOO LOW/NC"));
      display.display();  
    }
    else 
    {      
      byte chars = display.print(F("PEP: "));    
      chars += printFormattedNumber(roundDecimals(pep_dbm, 1) + attenuation, 1, 1, true, true);    
      chars += display.print(FS(str_dBm));
      display.setCursor(0, 20);                                   //************************************ gehe zur nächsten Zeile    
      chars = display.print(F("AM Mod: "));    
      chars += printFormattedNumber(modulation_index, 1, 1, true, true);    
      chars += display.print(F("%"));    
      display.display();
    }
}

// *************************************************************Funktion, Zeigt das Einstellungsmenü an, Frequenz kann geändert werden.*************************** 
void displayFrequencySettingsMenu() 
{
  displayloeschen();
  byte chars = display.print(F("set Frequenz:"));    
  display.setCursor(0, 20);  
  chars = printFormattedNumber(menu_freq, 4, 0, false, false);     
  chars += display.print(FS(str_MHz));   
  display.setCursor((display_col*6), 27);                       //Curser unter dem Text ######################################################
  display.print("-");
  display.setCursor(display_col, display_line); //***************** Setze den Cursur auf die gewünschte Position
  display.display();
}

// ****************************************************Funktion,  Zeige das Abschwächer Einstellmenu, Abschwächer einstellungen können geändert werden. *************** 
void displayAttenuationSettingsMenu() 
{
  displayloeschen();
  byte chars = display.print(F("Atten. profile"));     
  display.setCursor(0, 20);
  if (att_profile == 0)                                         //*********************************** Anwender  Profil
  {        
    chars = display.print(F("Custom:")); 
    chars+= printFormattedNumber(custom_attenuation, 2, 1, false, false);    
    chars+= display.print(FS(str_dBm)); 
  
    display.setCursor((display_col*6), 27);                     //Curser unter dem Text ######################################################  
    display.print("-");
    display.display();
  }
  else if (att_profile == 1)  
    chars = display.print(F("VAT-20W2+"));   
  else if (att_profile == 2)  
    chars = display.print(F("VAT-30W2+")); 
    display.setCursor(display_col, display_line);               //*************** Setze Cursor auf die gewünschte Position.
    display.display();
}

// **********************************************************Funktion,  Zeige das Kalibriemenue  **************************************************************
void displayCalibrateFrequencyMenu()
{
  displayloeschen();  
  byte chars = display.print(F("Kalibrations Menue"));    
  display.setCursor(0, 20);  
  chars = display.print(F("select a freq."));    
  display.display();
}

// ********************************************************** Zeige die Kalibrieroptionen
// ********************************************************** -2 = EXIT
// ********************************************************** -1 = Add new
// ********************************************************** 0-19 = calibrated and stored frequenz          
void displayCalibrateSelectedFreqMenu()
{  
  byte chars;
  unsigned int frequenz;
  
  displayloeschen();    
  if (selected_calibration_point == -2)
  {
    chars = display.print(F("EXIT"));
    display.setCursor(0, 20);  
    chars = display.print(F("Kalibration"));
    display.display();
  }
  else if (selected_calibration_point == -1)
  {
    chars = display.print(F("ENTER: Add new"));
    display.setCursor(0, 20);  
    chars = display.print(F("UP/DWN: Select"));
    display.display();
  }
  else
  {
    calibrationPoint point = calibrationData[selected_calibration_point];
    if (point.frequenz > 0)
    {
      chars = display.print("Freq: ");
      chars += printFormattedNumber(point.frequenz, 3, 0, false, false);     
      chars += display.print(FS(str_MHz));
      display.setCursor(0, 20);  
      chars = display.print("ENTER: delete");
      display.display();
    }
    else
    {
      chars = display.print(F("NO CALIBRATION DATA"));
    }
  } 
}

// ***********************************************Funktion, Bestätigen Sie die Löschbenachrichtigung.************************************************
void displayCalibrateConfirmDeleteMenu() 
{
  while (true) 
  {
    displayloeschen();    
    byte chars = display.print(F("Druecke <ENTER>"));    
    display.setCursor(0, 15);  
    chars = display.print(F("zum loeschen"));    
    display.setCursor(0, 30);
    chars = display.print(F("<UP>/<DOWN> zum"));    
    display.setCursor(0, 45);  
    chars = display.print(F("Abbruch"));    
    display.display();
    readButtonsAndWait(4000);
    if (current_menu != MENU_CALIBRIERUNG_BEST_LOESCHEN) 
      return;
  }
}

// ************************************ Anzeige des Kalibrierpunktes für gelöschte Benachrichtigung
void displayCalibrateDeletedMenuNotification() {
  
  displayloeschen();  
  byte chars = display.print(F("Kalibrations-"));    
  display.setCursor(0, 20);  
  chars = display.print(F("punkt geloescht!"));    
  display.display();
  readButtonsAndWait(2000);  
}

// **************************************Funktion,  Benutzer, mit dem Menu mit der gerade eingegebenen Frequenz auswählen.Das Signal muss eine bekannte Leistung haben..************* 
void displayCalibrateAskConnectMenu() 
{
  while (true)
  {
    displayloeschen();    
    display.print(F("Verbinde ein Signal"));    
    display.setCursor(0, 10);  
    byte chars = display.print(F("von "));   
    chars = printFormattedNumber(menu_freq, 3, 0, false, false);     
    chars += display.print(FS(str_MHz));   
    chars += display.print(F("  mit "));   
    display.setCursor(0, 20);  
    display.print(F("bekannte Leistung"));    
    display.setCursor(0, 30);  
    display.print(F("zwischen -10 dBm")); 
    display.setCursor(0, 40);  
    display.print(F("und -50 dBm zum"));    
    display.setCursor(0, 50);  
    display.print(F("Input anschliessen"));  
    display.display();    
    readButtonsAndWait(4000);
    if (current_menu != MENU_CALIBRIERUNG_FRAGE_VERBUN) 
      return;
      
    displayloeschen();    
    display.print(F("Enter den power"));    
    display.setCursor(0, 10);  
    display.print(F("im naechste Menu"));  
    display.setCursor(0, 20);  
    display.print(F("Drueck. <ENTER> zum"));    
    display.setCursor(0, 30);  
    display.print(F("fortfahr.or <UP>"));   
    display.setCursor(0, 40);  
    display.print(F("or <DOWN> to    "));    
    display.setCursor(0, 50);  
    display.print(F("Abbruch         "));   
    display.display();
    readButtonsAndWait(4000);
    if (current_menu != MENU_CALIBRIERUNG_FRAGE_VERBUN) 
      return;
  }  
}

// **********************************************************Funktion,  Displayabfrage zur Eingabe der bekannten Eingangsleistung ***********************************
void displayCalibrateEnterMeasuredPower() 
{
  displayloeschen();  
  byte chars = display.print(F("Enter input"));     
  
  display.setCursor(0, 20);     
  chars = display.print(F("power: ")); 
  chars+= printFormattedNumber(calibrated_power, 2, 1, true, false);    
  chars+= display.print(FS(str_dBm)); 

  display.setCursor((display_col*6), 27);               //Curser unter dem Text ######################################################
  display.print("-"); 
//  display.setCursor(display_col, display_line); //******************************* Setze Cursor auf die gewünschte Position.
  display.display();
}

// *********************************************************Funktion,  Display-Kalibrierung bereits vorhanden Benachrichtigung ****************************************
void displayCalibrationExistsNotification(int frequenz)
{
  displayloeschen();  
  byte chars = display.print(F("Freq: "));
  chars += display.print(menu_freq);    
  chars += display.print(FS(str_MHz));  
  display.setCursor(0, 20);  
  chars = display.print(F("ist schon vorhanden!"));
  display.display();
  delay(4000);  
}

// **************************************************************************Funktion, Anzeige nicht Kalibriert markierung links **************************************
void displayNoCalibrationSlotLeft()
{
  displayloeschen();  
  display.print(F("kein Platz für"));
  display.setCursor(0, 20);
  display.print(F("Kalibrierdaten")); 
  display.display();
  delay(4000);  
}

// *************************************************************************Funktion,  Display-Kalibrierung gespeicherte Werte ****************************************
void displayCalibrationSavedNotification()
{
  displayloeschen();
  display.print(F("Kalibrierung"));
  display.setCursor(0, 20);
  display.print(F("wurde gesichert!"));
  display.display();
  delay(2000);  
}

// ***************************************************************************Funktion, Anzeige des Begrüßungsbildschirms ********************************************* 
void displaySplashScreen() 
{
  displayloeschen();  
  display.setTextSize(2);
  display.print(F("DL3ED"));
  display.setTextSize(1);
  display.setCursor(0, 25);
  display.print(F("HF Power Meter"));
  display.setCursor(0, 40);
  display.print(F("Version 01-2021"));
  display.setCursor(0, 55);
  display.print(F("1 MHZ - 10 GHz"));
  display.display();
  delay(4000);

  displayloeschen();
  display.print(F("maximal Input"));
  display.setCursor(0, 15);
  display.print(F("Leist.10 dBm/10 mW"));  
  display.setCursor(0, 40);
  display.print(F("Drueck. Eing. 2 sec"));
  display.setCursor(0, 55);
  display.print(F("zum Kalibrieren"));  
  display.display();
  delay(4000);  
}
//*****************************************************************************************************************

void displayloeschen ()
  {
  display.clearDisplay();
  display.setTextColor(WHITE);            // , BLACK  );##############################
  display.setTextSize(1);
  display.setCursor (0,0);
  }  
